//+------------------------------------------------------------------+
//|                                   Indicator: FisherS Crosses.mq4 |
//|                                       Created with EABuilder.com |
//|                                        https://www.eabuilder.com |
//+------------------------------------------------------------------+
#property copyright "Created with EABuilder.com"
#property link      "https://www.eabuilder.com"
#property version   "1.00"
#property description ""
#property tester_indicator "Fisher_Yur4ik_Alert_v1"

#include <stdlib.mqh>
#include <stderror.mqh>

//--- indicator settings
#property indicator_chart_window
#property indicator_buffers 2

#property indicator_type1 DRAW_ARROW
#property indicator_width1 1
#property indicator_color1 0x00C4FF
#property indicator_label1 "FisherS BUY Alert"

#property indicator_type2 DRAW_ARROW
#property indicator_width2 1
#property indicator_color2 0x00AEFF
#property indicator_label2 "FisherS SELL Alert"

//--- indicator buffers
double Buffer1[];
double Buffer2[];

datetime time_alert; //used when sending alert
extern bool Audible_Alerts = true;
double myPoint; //initialized in OnInit

void myAlert(string type, string message)
  {
   int handle;
   if(type == "print")
      Print(message);
   else if(type == "error")
     {
      Print(type+" | FisherS Crosses @ "+Symbol()+","+IntegerToString(Period())+" | "+message);
     }
   else if(type == "order")
     {
     }
   else if(type == "modify")
     {
     }
   else if(type == "indicator")
     {
      Print(type+" | FisherS Crosses @ "+Symbol()+","+IntegerToString(Period())+" | "+message);
      if(Audible_Alerts) Alert(type+" | FisherS Crosses @ "+Symbol()+","+IntegerToString(Period())+" | "+message);
      handle = FileOpen("FisherS Crosses.txt", FILE_TXT|FILE_READ|FILE_WRITE|FILE_SHARE_READ|FILE_SHARE_WRITE, ';');
      if(handle != INVALID_HANDLE)
        {
         FileSeek(handle, 0, SEEK_END);
         FileWrite(handle, type+" | FisherS Crosses @ "+Symbol()+","+IntegerToString(Period())+" | "+message);
         FileClose(handle);
        }
     }
  }

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {   
   IndicatorBuffers(2);
   SetIndexBuffer(0, Buffer1);
   SetIndexEmptyValue(0, EMPTY_VALUE);
   SetIndexArrow(0, 241);
   SetIndexBuffer(1, Buffer2);
   SetIndexEmptyValue(1, EMPTY_VALUE);
   SetIndexArrow(1, 242);
   //initialize myPoint
   myPoint = Point();
   if(Digits() == 5 || Digits() == 3)
     {
      myPoint *= 10;
     }
   return(INIT_SUCCEEDED);
  }

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[])
  {
   int limit = rates_total - prev_calculated;
   //--- counting from 0 to rates_total
   ArraySetAsSeries(Buffer1, true);
   ArraySetAsSeries(Buffer2, true);
   //--- initial zero
   if(prev_calculated < 1)
     {
      ArrayInitialize(Buffer1, EMPTY_VALUE);
      ArrayInitialize(Buffer2, EMPTY_VALUE);
     }
   else
      limit++;
   
   //--- main loop
   for(int i = limit-1; i >= 0; i--)
     {
      if (i >= MathMin(5000-1, rates_total-1-50)) continue; //omit some old rates to prevent "Array out of range" or slow calculation   
      
      //Indicator Buffer 1
      if(iStochastic(NULL, PERIOD_CURRENT, 8, 3, 3, MODE_SMA, 0, MODE_MAIN, i) > 20
      && iStochastic(NULL, PERIOD_CURRENT, 8, 3, 3, MODE_SMA, 0, MODE_MAIN, i+1) < 20 //Stochastic Oscillator crosses above fixed value
      && iCustom(NULL, PERIOD_CURRENT, "Fisher_Yur4ik_Alert_v1", 10, false, false, true, false, "alert2.wav", 0, 1, i) > 0 //Fisher_Yur4ik_Alert_v1 > fixed value
      && iStochastic(NULL, PERIOD_CURRENT, 8, 3, 3, MODE_SMA, 0, MODE_MAIN, i) > iStochastic(NULL, PERIOD_CURRENT, 8, 3, 3, MODE_SMA, 0, MODE_SIGNAL, i) //Stochastic Oscillator > Stochastic Oscillator
      )
        {
         Buffer1[i] = Low[i]; //Set indicator value at Candlestick Low
         if(i == 0 && Time[0] != time_alert) { myAlert("indicator", "FisherS BUY Alert"); time_alert = Time[0]; } //Instant alert, only once per bar
        }
      else
        {
         Buffer1[i] = EMPTY_VALUE;
        }
      //Indicator Buffer 2
      if(iStochastic(NULL, PERIOD_CURRENT, 8, 3, 3, MODE_SMA, 0, MODE_MAIN, i) < 80
      && iStochastic(NULL, PERIOD_CURRENT, 8, 3, 3, MODE_SMA, 0, MODE_MAIN, i+1) > 80 //Stochastic Oscillator crosses below fixed value
      && iCustom(NULL, PERIOD_CURRENT, "Fisher_Yur4ik_Alert_v1", 10, false, false, true, false, "alert2.wav", 0, 2, i) < 0 //Fisher_Yur4ik_Alert_v1 < fixed value
      && iStochastic(NULL, PERIOD_CURRENT, 8, 3, 3, MODE_SMA, 0, MODE_MAIN, i) < iStochastic(NULL, PERIOD_CURRENT, 8, 3, 3, MODE_SMA, 0, MODE_SIGNAL, i) //Stochastic Oscillator < Stochastic Oscillator
      )
        {
         Buffer2[i] = High[i]; //Set indicator value at Candlestick High
         if(i == 0 && Time[0] != time_alert) { myAlert("indicator", "FisherS SELL Alert"); time_alert = Time[0]; } //Instant alert, only once per bar
        }
      else
        {
         Buffer2[i] = EMPTY_VALUE;
        }
     }
   return(rates_total);
  }
//+------------------------------------------------------------------+